// rhsBitmapScale
// ----------------------------------------------------------------------------
// Example:
// BitmapScale(Image1.Picture.Bitmap, NewWidth, NewHeight);

// If 0 is passed to NewWidth or NewHeight, the corresponding value is
// automatically calculated in the correct aspect ratio.
// Example:
// BitmapScale(Image1.Picture.Bitmap, 0, NewHeight);
// ----------------------------------------------------------------------------

unit rhsBitmapScale;

{$mode objfpc}{$H+}

interface

uses
  Graphics, IntfGraphics, LazCanvas;

procedure BitmapScale(BM: TCustomBitmap; NewWidth, NewHeight: Integer);

implementation

procedure BitmapScale(BM: TCustomBitmap; NewWidth, NewHeight: Integer);
var
  Source: TLazIntfImage = nil;
  Dest: TLazCanvas = nil;
begin
  if (NewWidth > BM.Width) or (NewHeight > BM.Height) then
    Exit;

  if NewHeight = 0 then
    NewHeight := Trunc(NewWidth / (BM.Width / BM.Height));

  if NewWidth = 0 then
    NewWidth := Trunc(NewHeight / (BM.Height / BM.Width));

  try
    // Create the Source-LazIntfImage
    Source := BM.CreateIntfImage;
    // Create the Destination-LazCanvas
    Dest := TLazCanvas.Create(Source);
    // Execute the Canvas.StretchDraw
    Dest.StretchDraw(0, 0, NewWidth, NewHeight, Source);
    // Reload the stretched Source into the Bitmap
    BM.LoadFromIntfImage(Source);
    BM.SetSize(NewWidth, NewHeight);
  finally
    Dest.Free;
    Source.Free;
  end;
end;

end.
